@@ -38,19 +38,19 @@ pub fn camel_case_name(name: &str) -> String {
 pub fn snake_case_name(name: &str) -> String {
     let mut name = name.to_string();
     name = name.replace('-', "_");
-    let it = name.chars().peekable();
     let mut result = String::new();
-    let mut started = false;
-    for c in it {
+    let mut prev_upper = false;
+
+    for c in name.chars() {
         if c.is_uppercase() {
-            if !result.is_empty() && !started && !result.ends_with('_') {
+            if !result.is_empty() && !prev_upper && !result.ends_with('_') {
                 result.push('_');
-                started = true;
             }
-            result.push_str(c.to_lowercase().collect::<String>().as_str());
+            result.push_str(&c.to_lowercase().to_string());
+            prev_upper = true;
         } else {
             result.push(c);
-            started = false;
+            prev_upper = false;
         }
     }
     result
@@ -147,6 +147,9 @@ fn generate_resource_type_ref_accessors(
             .unwrap_or(false);
         if let Some(doc) = param.doc.as_ref() {
             lines.extend(generate_doc(doc, 1, config));
+        } else {
+            // Generate default getter documentation
+            lines.push(format!("    /// Get the {} value.\n", param.name));
         }
         let field_type = camel_case_name(id);
         let mut ret_type = field_type.to_string();
@@ -160,7 +163,7 @@ fn generate_resource_type_ref_accessors(
         } else {
             None
         };
-        if !param.required {
+        if config.nillable(param) {
             ret_type = format!("Option<{}>", ret_type);
         }
         let accessor_name = if let Some(rename_fn) = config.param_accessor_rename.as_ref() {
@@ -188,7 +191,7 @@ fn generate_resource_type_ref_accessors(
             accessor_name,
             ret_type
         ));
-        if param.required {
+        if !config.nillable(param) {
             if let Some(map_fn) = map_fn {
                 lines.push(format!(
                     "        {}({}(self.{}.clone())\n",
@@ -219,6 +222,9 @@ fn generate_resource_type_ref_accessors(
             lines.push("    #[deprecated]".to_string());
         }
 
+        // Add setter documentation
+        lines.push(format!("    /// Set the {} value.\n", param.name));
+
         lines.push(format!(
             "    {}fn set_{}(&mut self, value: {}) {{\n",
             if visibility.is_empty() {
@@ -230,7 +236,7 @@ fn generate_resource_type_ref_accessors(
             ret_type
         ));
 
-        if param.required {
+        if !config.nillable(param) {
             lines.push(format!(
                 "        self.{} = value.url().clone();\n",
                 field_name
@@ -374,7 +380,7 @@ fn param_rust_type(
         param_type = format!("Vec<{}>", param_type);
     }
 
-    if !param.required {
+    if config.nillable(param) {
         param_type = format!("Option<{}>", param_type);
     }
 
@@ -450,7 +456,7 @@ fn generate_representation_struct_json(
         ));
     }
 
-    let derive_default = input.params.iter().all(|x| !x.required);
+    let derive_default = input.params.iter().all(|x| config.nillable(x));
 
     lines.push(
         "#[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]\n".to_string(),
@@ -580,7 +586,7 @@ fn rust_type_for_response(
                     ret.push(param_type);
                 }
                 if ret.len() == 1 {
-                    ret[0].clone()
+                    ret.into_iter().next().unwrap()
                 } else {
                     format!("({})", ret.join(", "))
                 }
@@ -599,7 +605,7 @@ fn rust_type_for_response(
             ret.push(param_type);
         }
         if ret.len() == 1 {
-            ret[0].clone()
+            ret.into_iter().next().unwrap()
         } else {
             format!("({})", ret.join(", "))
         }
@@ -640,9 +646,9 @@ fn format_arg_doc(name: &str, doc: Option<&crate::ast::Doc>, config: &Config) ->
     lines
 }
 
-fn apply_map_fn(map_fn: Option<&str>, ret: &str, required: bool) -> String {
+fn apply_map_fn(map_fn: Option<&str>, ret: &str, nillable: bool) -> String {
     if let Some(map_fn) = map_fn {
-        if required {
+        if !nillable {
             if map_fn.starts_with('|') {
                 format!("({})({})", map_fn, ret)
             } else {
@@ -812,6 +818,20 @@ fn serialize_representation_def(
     lines
 }
 
+/// Generate code for a WADL method (HTTP GET, POST, PUT, DELETE, etc.).
+///
+/// This function generates Rust methods that correspond to HTTP operations defined
+/// in the WADL specification. It creates both representation-based methods and
+/// WADL-specific methods when applicable.
+///
+/// # Arguments
+/// * `input` - The WADL method definition containing HTTP method details
+/// * `parent_id` - The ID of the parent resource type
+/// * `config` - Configuration options for code generation
+/// * `options_names` - Mapping of option sets to their generated enum names
+///
+/// # Returns
+/// A vector of strings containing the generated Rust code lines
 fn generate_method(
     input: &Method,
     parent_id: &str,
@@ -831,6 +851,22 @@ fn generate_method(
     lines
 }
 
+/// Generate Rust code for a WADL method that retrieves WADL descriptions.
+///
+/// This function creates methods that fetch WADL representations from endpoints.
+/// These methods are generated when a resource supports returning its own WADL
+/// description (typically for dynamic API discovery). The generated method will:
+/// - Build a request with appropriate WADL accept headers
+/// - Parse the returned WADL XML into AST structures
+/// - Return the corresponding resource definition
+///
+/// # Arguments
+/// * `input` - The WADL method definition
+/// * `parent_id` - The ID of the parent resource type (used for method naming)
+/// * `config` - Configuration for code generation (async/sync mode)
+///
+/// # Returns
+/// A vector of strings containing the generated Rust method for WADL retrieval
 fn generate_method_wadl(input: &Method, parent_id: &str, config: &Config) -> Vec<String> {
     let mut lines = vec![];
 
@@ -842,6 +878,17 @@ fn generate_method_wadl(input: &Method, parent_id: &str, config: &Config) -> Vec
 
     let async_prefix = if config.r#async { "async " } else { "" };
 
+    // Add documentation for WADL method
+    lines.push("    /// Retrieve the WADL description for this resource.\n".to_string());
+    lines.push("    ///\n".to_string());
+    lines.push("    /// This method fetches the WADL (Web Application Description Language) specification\n".to_string());
+    lines.push(
+        "    /// for the current resource, allowing for runtime API discovery.\n".to_string(),
+    );
+    lines.push("    ///\n".to_string());
+    lines.push("    /// # Returns\n".to_string());
+    lines.push("    /// Returns the `wadl::ast::Resource` definition on success, or an error if the request fails.\n".to_string());
+
     lines.push(format!("    pub {}fn {}_wadl<'a>(&self, client: &'a dyn {}) -> std::result::Result<wadl::ast::Resource, wadl::Error> {{\n", async_prefix, name, config.client_trait_name()));
 
     lines.push("        let mut url_ = self.url().clone();\n".to_string());
@@ -900,6 +947,23 @@ fn generate_method_wadl(input: &Method, parent_id: &str, config: &Config) -> Vec
     lines
 }
 
+/// Generate Rust code for a WADL method that handles representations.
+///
+/// This function creates the actual implementation of HTTP methods (GET, POST, PUT, DELETE, etc.)
+/// that handle request/response representations. It generates type-safe Rust methods that:
+/// - Accept appropriate parameters based on the WADL definition
+/// - Build HTTP requests with proper headers and query parameters
+/// - Serialize request bodies according to the representation type
+/// - Deserialize responses into strongly-typed Rust structures
+///
+/// # Arguments
+/// * `input` - The WADL method definition containing request/response specifications
+/// * `parent_id` - The ID of the parent resource type (used for method naming)
+/// * `config` - Configuration for code generation (async/sync, visibility, etc.)
+/// * `options_names` - Mapping of option sets to their generated enum names
+///
+/// # Returns
+/// A vector of strings containing the generated Rust method implementation
 fn generate_method_representation(
     input: &Method,
     parent_id: &str,
@@ -965,8 +1029,24 @@ fn generate_method_representation(
             .flatten(),
     );
 
-    for doc in &input.docs {
-        lines.extend(generate_doc(doc, 1, config));
+    // Generate documentation for the method
+    if input.docs.is_empty() {
+        // Generate a default docstring based on the HTTP method and resource
+        let method_verb = match input.name.as_str() {
+            "GET" => "Retrieve",
+            "POST" => "Create",
+            "PUT" => "Update",
+            "DELETE" => "Delete",
+            "PATCH" => "Partially update",
+            "HEAD" => "Get headers for",
+            _ => "Perform operation on",
+        };
+        lines.push(format!("    /// {} the resource.\n", method_verb));
+        lines.push("    ///\n".to_string());
+    } else {
+        for doc in &input.docs {
+            lines.extend(generate_doc(doc, 1, config));
+        }
     }
 
     if !params.is_empty() {
@@ -1003,6 +1083,17 @@ fn generate_method_representation(
 
         lines.extend(format_arg_doc(param_name, param.doc.as_ref(), config));
     }
+
+    // Add returns documentation
+    if ret_type != "()" {
+        lines.push("    ///\n".to_string());
+        lines.push("    /// # Returns\n".to_string());
+        lines.push(format!(
+            "    /// Returns `{}` on success, or an error if the request fails.\n",
+            ret_type
+        ));
+    }
+
     line.push_str(") -> std::result::Result<");
     line.push_str(ret_type.as_str());
 
@@ -1169,7 +1260,7 @@ fn generate_method_representation(
         } else if return_types.len() == 1 {
             format!(
                 "Ok({})",
-                apply_map_fn(map_fn.as_deref(), &return_types[0].0, return_types[0].1)
+                apply_map_fn(map_fn.as_deref(), &return_types[0].0, !return_types[0].1)
             )
         } else {
             let v = format!(
@@ -1180,7 +1271,7 @@ fn generate_method_representation(
                     .collect::<Vec<_>>()
                     .join(", ")
             );
-            format!("Ok({})", apply_map_fn(map_fn.as_deref(), &v, true))
+            format!("Ok({})", apply_map_fn(map_fn.as_deref(), &v, false))
         }
     };
 
@@ -1192,7 +1283,7 @@ fn generate_method_representation(
                 ParamStyle::Header => {
                     if !param.links.is_empty() {
                         let r = &param.links[0].resource_type.as_ref().unwrap();
-                        if param.required {
+                        if !config.nillable(param) {
                             return_types.push((
                                 format!(
                                     "{}(resp.headers().get(\"{}\")?.to_str()?.parse().unwrap())",
@@ -1416,6 +1507,9 @@ pub struct Config {
 
     /// Convert a string to a multipart Part, given a type name and value
     pub convert_to_multipart: Option<Box<dyn Fn(&str, &str) -> Option<String>>>,
+
+    /// Check whether a parameter can be nil
+    pub nillable_param: Option<Box<dyn Fn(&Param) -> bool>>,
 }
 
 impl Config {
@@ -1427,6 +1521,15 @@ impl Config {
             "wadl::blocking::Client"
         }
     }
+
+    /// Check whether the parameter is can be nil
+    pub fn nillable(&self, param: &Param) -> bool {
+        if let Some(nillable_param) = self.nillable_param.as_ref() {
+            nillable_param(param)
+        } else {
+            !param.required
+        }
+    }
 }
 
 fn enum_rust_value(option: &str) -> String {
@@ -2130,18 +2233,21 @@ This is another test"#;
 
     #[test]
     fn test_apply_map_fn() {
-        assert_eq!(apply_map_fn(None, "x", true), "x".to_string());
-        assert_eq!(apply_map_fn(Some("Some"), "x", true), "Some(x)".to_string());
+        assert_eq!(apply_map_fn(None, "x", false), "x".to_string());
         assert_eq!(
             apply_map_fn(Some("Some"), "x", false),
+            "Some(x)".to_string()
+        );
+        assert_eq!(
+            apply_map_fn(Some("Some"), "x", true),
             "x.map(Some)".to_string()
         );
         assert_eq!(
-            apply_map_fn(Some("|y|y+1"), "x", true),
+            apply_map_fn(Some("|y|y+1"), "x", false),
             "(|y|y+1)(x)".to_string()
         );
         assert_eq!(
-            apply_map_fn(Some("|y|y+1"), "x", false),
+            apply_map_fn(Some("|y|y+1"), "x", true),
             "x.map(|y|y+1)".to_string()
         );
     }
@@ -2162,6 +2268,8 @@ This is another test"#;
         let config = Config::default();
         let lines = generate_method(&input, "bar", &config, &HashMap::new());
         assert_eq!(lines, vec![
+        "    /// Retrieve the resource.\n".to_string(),
+        "    ///\n".to_string(),
         "    pub fn foo<'a>(&self, client: &'a dyn wadl::blocking::Client) -> std::result::Result<(), wadl::Error> {\n".to_string(),
         "        let mut url_ = self.url().clone();\n".to_string(),
         "\n".to_string(),
